# START SSEG-CONV
# Seven Segment Hex Converter
#	input: $a0(4 bit number)
#	output: $v0(sseg code)
#   overwrites:
#       $t9
#       $t8
sseg-conv:
	andi $a0 , $a0 , 0x0000000F	# force $a0 to be only 4 bits(bit masking)
	ori $t8 , $0 , 1	# store 1 used for later addition
	or $t9 , $0 , $0	# clear $t9

	beq $a0 , $t9 , sseg-zero	# if $a0(our parameter) is zero, jump to zero label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-one	# if $a0(our parameter) is zero, jump to one label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-two	# if $a0(our parameter) is zero, jump to two label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-three	# if $a0(our parameter) is zero, jump to three label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-four	# if $a0(our parameter) is zero, jump to four label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-five	# if $a0(our parameter) is zero, jump to five label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-six	# if $a0(our parameter) is zero, jump to six label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-seven	# if $a0(our parameter) is zero, jump to seven label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-eight	# if $a0(our parameter) is zero, jump to eight label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-nine	# if $a0(our parameter) is zero, jump to nine label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-a	# if $a0(our parameter) is zero, jump to a label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-b	# if $a0(our parameter) is zero, jump to b label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-c	# if $a0(our parameter) is zero, jump to c label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-d	# if $a0(our parameter) is zero, jump to d label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-e	# if $a0(our parameter) is zero, jump to e label
	addu $t9 , $t9 , $t8	# increment $t9 by one in the branch delay slot, improves performance over time
	beq $a0 , $t9 , sseg-f	# if $a0(our parameter) is zero, jump to f label
	nop	# final jump does not need to increment
	
	# this code will only get accessed if the number fails(should not happen, ever)
	or $v0 , $0 , $0	# clear $v0
	jr $ra  # return to where we were when we jal to sseg-conv by using $ra(return address)
	nop # nop in branch delay slot

sseg-zero:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b11000000	# sseg code for 0 character, using branch delay slot for max efficiency

sseg-one:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b11111001	# sseg code for 1 character, using branch delay slot for max efficiency

sseg-two:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10100100	# sseg code for 2 character, using branch delay slot for max efficiency

sseg-three:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10110000	# sseg code for 3 character, using branch delay slot for max efficiency

sseg-four:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10011001	# sseg code for 4 character, using branch delay slot for max efficiency

sseg-five:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10010010	# sseg code for 5 character, using branch delay slot for max efficiency

sseg-six:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10000010	# sseg code for 6 character, using branch delay slot for max efficiency

sseg-seven:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b11111000	# sseg code for 7 character, using branch delay slot for max efficiency

sseg-eight:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10000000	# sseg code for 8 character, using branch delay slot for max efficiency

sseg-nine:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10010000	# sseg code for 9 character, using branch delay slot for max efficiency

sseg-a:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10001000	# sseg code for a character, using branch delay slot for max efficiency

sseg-b:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10000011	# sseg code for b character, using branch delay slot for max efficiency

sseg-c:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b11000110	# sseg code for c character, using branch delay slot for max efficiency

sseg-d:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10100001	# sseg code for d character, using branch delay slot for max efficiency

sseg-e:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10000110	# sseg code for e character, using branch delay slot for max efficiency

sseg-f:
	jr $ra	# jump to $ra(will have last position in program)
	ori $v0 , $0 , 0b10001110	# sseg code for f character, using branch delay slot for max efficiency
# END of SSEG-CONV